#!/usr/bin/env python
#
# Some tests for Chewy version
#

""" Unit tests for chewy.Version """

# Standard imports
import os
import sys
import unittest

sys.path.insert(0, '..')

# Project specific imports
import chewy


class ChewyVersionTester(unittest.TestCase):
    '''Unit tests for [???]'''

    def setUp(self):
        pass


    def test_version(self):
        ver1 = chewy.Version('1.2')
        ver2 = chewy.Version('1.3')
        self.assertTrue(ver1 < ver2)
        self.assertFalse(ver1 > ver2)
        self.assertFalse(ver1 == ver2)

        ver1 = chewy.Version('2.4')
        ver2 = chewy.Version('5.1')
        self.assertTrue(ver1 < ver2)
        self.assertFalse(ver1 > ver2)
        self.assertFalse(ver1 == ver2)

        ver1 = chewy.Version('05. 1')
        ver2 = chewy.Version('  20.4')
        self.assertTrue(ver1 < ver2)
        self.assertFalse(ver1 > ver2)
        self.assertFalse(ver1 == ver2)

    def test_exception(self):
        try:
            chewy.Version('5.h6')
            self.assertTrue(False)
        except chewy.VersionError:
            self.assertTrue(True)

        try:
            chewy.Version('')
            self.assertTrue(False)
        except chewy.VersionError:
            self.assertTrue(True)

        try:
            chewy.Version('56')
            self.assertTrue(False)
        except chewy.VersionError:
            self.assertTrue(True)

        try:
            chewy.Version(None)
            self.assertTrue(False)
        except chewy.VersionError:
            self.assertTrue(True)


if __name__ == '__main__':
    suite = unittest.makeSuite(ChewyVersionTester)
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    if result.wasSuccessful() != True:
        sys.exit(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Testable code for Chewy
#
#


from chewy.manifest import Manifest, ManifestError
from chewy.module import Module, ModuleStatus, ModuleError
from chewy.version import Version, VersionError
from chewy.session import Session
from chewy.session import HttpEndpoint
from chewy.fancy_grid import FancyGrid

import shutil
import os
import fnmatch

# Set PEP 396 module version
__version__="0.4"

EXPECTED_CMAKE_MODULES_PATH = 'cmake/modules'


class NoMetaError(RuntimeError):
    pass


def modules_dir_lookup(start_path = os.getcwd()):
    while start_path != '/':
        try_mod_dir = os.path.join(start_path, EXPECTED_CMAKE_MODULES_PATH)
        if os.path.isdir(try_mod_dir):
            return try_mod_dir
        else:
            start_path = os.path.dirname(start_path)
    raise RuntimeError("Unable to find CMake modules directory `{}'".format(EXPECTED_CMAKE_MODULES_PATH))


def modules_lookup(modules_dir):
    ''' Return: list<string> or [] '''
    result = []
    # TODO: Check file is module by internal X-Chewy tags
    for root, dirs, files in os.walk(modules_dir):
        result += [os.path.join(root, x) for x in fnmatch.filter(files, '*.cmake')]
    return result


def open_module(module_file):
    with open(module_file, 'rt') as f:
        content = f.read()
        return Module(content)


def collect_installed_modules(modules_dir):
    ''' Return: a dict of `repobase` to a list of `Module`, status, None '''

    # Group installed modules by unique repobase
    # dict<list<Modules>, status, remote_version>
    mod_list = {}
    for module_file in modules_lookup(modules_dir):
        try:
            mod = open_module(module_file)
            if not mod.repobase in mod_list:
                mod_list[mod.repobase] = []
            mod_list[mod.repobase].append(ModuleStatus(mod))
        except (NoMetaError, ModuleError, IOError) as ex:
            # TODO: logging
            #log.ewarn('Module {} has error: {}'.format(module_file, ex))
            continue
    return mod_list


def find(seq, pred):
    return next(
        filter(lambda x: pred(x), seq)
      , None
      )


class PathError(RuntimeError):
    pass


# TODO: unittests
def sandbox_path(prefix, path):
    assert(
        'We expect prefix is an absolute path in the function by security reason'
        and prefix == os.path.abspath(prefix)
      )
    abspath = os.path.abspath(os.path.join(prefix, path))
    if not abspath.startswith(prefix):
        raise PathError(
            'Relative pathname {} trying to pass the sandbox {}'.format(path, abspath)
          )
    return abspath


# TODO: unittests
def copytree(src, dst, symlinks=False, ignore=None):
    '''
        shutil.copytree can't copy to existed destenition. We need the own analogue
    '''
    assert(src and dst)
    for path, dirs, files in os.walk(src, topdown=True):
        base = path[ len(src) if path.startswith(src) else 0 : ].strip('/')
        for d in dirs:
            dirname = os.path.join(dst, base, d)
            if not os.path.exists(dirname):
                os.mkdir(dirname)
            elif not os.path.isdir(dirname):
                RuntimeError("Path {} is exist and it isn't directory as expected".format(dirnmae))
        for f in files:
            shutil.copy(os.path.join(path, f), os.path.join(dst, base, f))



#!/usr/bin/env python
#
# Some tests for chewy
#

""" Unit tests for Chewy """

# Standard imports
import os
import sys
import unittest

sys.path.insert(0, '..')

# Project specific imports
import chewy


class ChewyTester(unittest.TestCase):
    '''Unit tests for [???]'''

    def setUp(self):
        pass

    def test_smth(self):
        '''Add test code here'''
        pass

if __name__ == '__main__':
#    unittest.main()
    suite = unittest.TestLoader().loadTestsFromTestCase(ChewyTester)
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    if result.wasSuccessful() != True:
        sys.exit(1)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Chewy MetaInfo helpers
#

import re


_X_META_REGEX = re.compile('^\s*#\s+X-Chewy-([^:]+):\s*(.*)\s*$')

PATH = 'Path'
VERSION = 'Version'
DESCRIPTION = 'Description'
ADDON = 'AddonFile'
REPOBASE = 'RepoBase'


def parse(file_content):
    '''Try to parse given file content and get meta fields as list of KVP'''
    lines = file_content.split('\n')
    meta = []
    for line in lines:
        m = _X_META_REGEX.search(line)
        if m:
            meta.append((m.group(1).strip(), m.group(2).strip()))
    return meta




#!/usr/bin/env python
#
# Some tests for Chewy manifest
#

""" Unit tests for chewy.Manifest """

# Standard imports
import os
import sys
import unittest

sys.path.insert(0, '..')

# Project specific imports
import chewy
import chewy.manifest

_test_case_1 = '''
# X-Chewy-RepoBase: https://raw.github.com/mutanabbi/chewy-cmake-rep/master/
test.cmake 2.0 sample+description
'''
_test_case_2 = '''
# X-Chewy-RepoBase: https://raw.github.com/mutanabbi/chewy-cmake-rep/master/
test.cmake 2.0 sample+description some.file another.file
'''
_invalid_case_1 = '''
# X-Chewy-RepoBase: https://raw.github.com/mutanabbi/chewy-cmake-rep/master/
'''


class ChewyModuleTester(unittest.TestCase):
    '''Unit tests for chewy.Manifest'''

    def setUp(self):
        pass


    def test_empty_manifest(self):
        try:
            result = chewy.Manifest("")
            self.assertTrue(False)
        except chewy.NoMetaError:
            self.assertTrue(True)


    def test_manifest(self):
        manifest = chewy.Manifest(_test_case_1)
        self.assertEqual(manifest.repobase, 'https://raw.github.com/mutanabbi/chewy-cmake-rep/master/')
        self.assertEqual(len(manifest.modules), 1)
        self.assertEqual(manifest.modules[0].path, 'test.cmake')
        self.assertEqual(manifest.modules[0].version, chewy.Version('2.0'))
        self.assertEqual(manifest.modules[0].description, 'sample description')

    def test_manifest_with_addons(self):
        manifest = chewy.Manifest(_test_case_2)
        self.assertEqual(manifest.repobase, 'https://raw.github.com/mutanabbi/chewy-cmake-rep/master/')
        self.assertEqual(len(manifest.modules), 1)
        self.assertEqual(manifest.modules[0].path, 'test.cmake')
        self.assertEqual(manifest.modules[0].version, chewy.Version('2.0'))
        self.assertEqual(manifest.modules[0].description, 'sample description')
        self.assertEqual(len(manifest.modules[0].addons), 2)
        self.assertEqual(manifest.modules[0].addons[0], 'some.file')
        self.assertEqual(manifest.modules[0].addons[1], 'another.file')


if __name__ == '__main__':
    suite = unittest.makeSuite(ChewyModuleTester)
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    if result.wasSuccessful() != True:
        sys.exit(1)




#!/usr/bin/env python
#
# Some tests for Chewy module
#

""" Unit tests for chewy.Module """

# Standard imports
import os
import sys
import unittest

sys.path.insert(0, '..')

# Project specific imports
import chewy

_test_case_1 = '''
# X-Chewy-RepoBase: https://raw.github.com/mutanabbi/chewy-cmake-rep/master/
# X-Chewy-Path: AddBoostTests.cmake
# X-Chewy-Version: 2.0
# X-Chewy-Description: Integrate Boost unit tests into CMake infrastructure
'''
_invalid_case_1 = '''
# X-Chewy-RepoBase: https://raw.github.com/mutanabbi/chewy-cmake-rep/master/
'''
_invalid_case_2 = '''
# X-Chewy-RepoBase: https://raw.github.com/mutanabbi/chewy-cmake-rep/master/
# X-Chewy-Path: AddBoostTests.cmake
'''
_invalid_case_3 = '''
# X-Chewy-RepoBase: https://raw.github.com/mutanabbi/chewy-cmake-rep/master/
# X-Chewy-Path: AddBoostTests.cmake
# X-Chewy-Version: 2.0
'''


class ChewyModuleTester(unittest.TestCase):
    '''Unit tests for chewy.Module'''

    def setUp(self):
        pass


    def test_empty_module(self):
        try:
            result = chewy.Module("")
            self.assertTrue(False)
        except chewy.NoMetaError:
            self.assertTrue(True)


    def test_invalid_module_1(self):
        try:
            result = chewy.Module(_invalid_case_1)
            self.assertTrue(False)
        except chewy.ModuleError:
            self.assertTrue(True)


    def test_invalid_module_2(self):
        try:
            result = chewy.Module(_invalid_case_2)
            self.assertTrue(False)
        except chewy.ModuleError:
            self.assertTrue(True)


    def test_invalid_module_3(self):
        try:
            result = chewy.Module(_invalid_case_3)
            self.assertTrue(False)
        except chewy.ModuleError:
            self.assertTrue(True)


    def test_module(self):
        mod = chewy.Module(_test_case_1)
        self.assertEqual(mod.repobase, 'https://raw.github.com/mutanabbi/chewy-cmake-rep/master/')
        self.assertEqual(mod.path, 'AddBoostTests.cmake')
        self.assertEqual(mod.version, chewy.Version('2.0'))
        self.assertTrue(0 < len(mod.description))


    def test_module_alt(self):
        mod = chewy.Module(chewy.Module.PiecewiseConstruct('repobase', 'test.cmake', '1.0', 'description'))
        self.assertEqual(mod.repobase, 'repobase')
        self.assertEqual(mod.path, 'test.cmake')
        self.assertEqual(mod.version, chewy.Version('1.0'))
        self.assertEqual(mod.description, 'description')


class ChewyModuleStatusTester(unittest.TestCase):
    '''Unit tests for chewy.ModuleStatus'''

    def setUp(self):
        pass


    def test_module_status_1(self):
        mod = chewy.Module(_test_case_1)
        status = chewy.ModuleStatus(mod)
        status.set_remote_version(chewy.Version('1.0'))
        self.assertFalse(status.needs_update())


    def test_module_status_2(self):
        mod = chewy.Module(_test_case_1)
        status = chewy.ModuleStatus(mod)
        status.set_remote_version(chewy.Version('2.0'))
        self.assertFalse(status.needs_update())


    def test_module_status_3(self):
        mod = chewy.Module(_test_case_1)
        status = chewy.ModuleStatus(mod)
        status.set_remote_version(chewy.Version('2.1'))
        self.assertTrue(status.needs_update())


    def test_module_status_4(self):
        mod = chewy.Module(_test_case_1)
        status = chewy.ModuleStatus(mod)
        status.set_remote_version(chewy.Version('3.5'))
        self.assertTrue(status.needs_update())


if __name__ == '__main__':
    module_suite = unittest.makeSuite(ChewyModuleTester)
    status_suite = unittest.makeSuite(ChewyModuleStatusTester)
    module_result = unittest.TextTestRunner(verbosity=2).run(module_suite)
    status_result = unittest.TextTestRunner(verbosity=2).run(status_suite)
    if module_result.wasSuccessful() != True or status_result.wasSuccessful() != True:
        sys.exit(1)




#!/usr/bin/env python
#
# Some tests for chewy
#

""" Unit tests for chewy.FancyGrid """

# Standard imports
import os
import sys
import unittest

sys.path.insert(0, '..')

# Project specific imports
import chewy

_test_case_1=[[1], [2], [3]]
_expect_1='''\
 1
 2
 3
'''
_test_case_2=[(1, 'uno'), (2, 'dos'), (3, 'tres')]
_expect_2='''\
1   uno
2   dos
3   tres
'''
_test_case_3=[(1, '-', 'uno'), (2, '-', 'dos'), (3, '-', 'tres')]
_expect_3='''\
1  -   uno
2  -   dos
3  -   tres
'''
_test_case_4=[(1, 'uno', '-'), (2, 'dos', '-'), (3, 'tres', '-')]
_expect_4='''\
1  uno    -
2  dos    -
3  tres   -
'''

class FancyGridTester(unittest.TestCase):
    '''Unit tests for chewy.FancyGrid'''

    def setUp(self):
        pass

    def test_one(self):
        result = str(chewy.FancyGrid(_test_case_1))
        self.assertEqual(result, _expect_1)

    def test_two(self):
        result = str(chewy.FancyGrid(_test_case_2))
        self.assertEqual(result, _expect_2)

    def test_three(self):
        result = str(chewy.FancyGrid(_test_case_3))
        self.assertEqual(result, _expect_3)

    def test_four(self):
        result = str(chewy.FancyGrid(_test_case_4))
        self.assertEqual(result, _expect_4)


if __name__ == '__main__':
    suite = unittest.makeSuite(FancyGridTester)
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    if result.wasSuccessful() != True:
        sys.exit(1)




#!/usr/bin/env python
#
# Some tests for chewy.Session and chewy.Factory
#

""" Unit tests for chewy.Session """

# Standard imports
import http.client
import os
import sys
import unittest
from mock import patch

sys.path.insert(0, '..')

# Project specific imports
import chewy

_TEST_REPO_URL = 'http://localhost/chewy/repo'
_TEST_FILE_URL = 'http://localhost/chewy/repo/test.cmake'
_TEST_FILE_CONTENT = '''
# Sample
'''


class FakeHTTPResponse(object):
    def __init__(self):
        self.status = http.client.OK

    def read(self):
        class FakeData(object):
            def decode(self, encode):
                return _TEST_FILE_CONTENT
        return FakeData()


class FakeHTTPConnection(object):
    def __init__(self, suite):
        self.status = http.client.OK

    def connect(self):
        pass

    def close(self):
        pass

    def request(self, method, filename):
        pass

    def getresponse(self):
        return FakeHTTPResponse()



class ChewySessionTester(unittest.TestCase):
    '''Unit tests for [???]'''

    def setUp(self):
        self.connection = FakeHTTPConnection(self)


    @patch('http.client.HTTPConnection')
    def test_smth(self, mock_connection):
        # Configure mock HTTP connection
        mock_connection.return_value = self.connection

        # Get session for given repobase URL
        ep = chewy.HttpEndpoint(_TEST_REPO_URL)
        sf = chewy.session.Factory()
        session = sf.get_session(ep)
        contents = session.retrieve_remote_file(_TEST_FILE_URL)
        #mock_connection.request.assert_called_with('GET', _TEST_FILE_URL)
        return


if __name__ == '__main__':
    suite = unittest.makeSuite(ChewySessionTester)
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    if not result.wasSuccessful():
        sys.exit(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

import http.client
import os
import socket
import urllib.parse

import chewy


MANIFEST_PATH = 'manifest'
_HEADERS = {
    'User-Agent': 'Chewy 0.1'
  , 'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8'
  , 'Accept-Language': 'en-us,en;q=0.7,ru;q=0.3'
  , 'DNT': 1
  , 'Connection': 'keep-alive'
  }


class NetworkError(RuntimeError):
    pass



class HttpEndpoint(urllib.parse.SplitResult):
    ''' Class to represent a HTTP endpoint to contact'''

    is_ssl = False

    def __new__(cls, url):
        return super(HttpEndpoint, cls).__new__(cls, *urllib.parse.urlsplit(url))

    def __init__(self, url):
        '''Parse the URL given and construct an endpoint instance'''
        self.url = url
        if self.scheme == 'http':
            self.is_ssl = False
        elif self.scheme == 'https':
            self.is_ssl = True
        else:
            raise RuntimeError("Unsupported scheme in URL `{}'".format(url))


class Factory(object):
    def __init__(self):
        self.__cp = {}

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        for domain in self.__cp:
            self.__cp[domain].close()

    def get_session(self, ep):
        key = ep.scheme, ep.hostname, ep.port
        if key in self.__cp:
            return self.__cp[key]
        else:
            cs = Session(ep)
            self.__cp[key] = cs
            return cs



class Session(object):
    '''Class to access a remote CHEWY repository'''

    def __init__(self, endpoint):
        '''Connect the endpoint given and be ready for files transfer'''
        if not isinstance(endpoint, HttpEndpoint):
            raise TypeError('Endpoint to contact has invalid type')

        self.__ep = endpoint
        import socket
        try:
            if self.__ep.is_ssl:
                self.__connection = http.client.HTTPSConnection(self.__ep.hostname, self.__ep.port)
            else:
                self.__connection = http.client.HTTPConnection(self.__ep.hostname, self.__ep.port)
            self.__connection.connect()
        except socket.error as ex:
            raise NetworkError("hostname: `{}'; port: `{}': {}".format(self.__ep.hostname, self.__ep.port, ex))


    def close(self):
        self.__connection.close()


    def get_manifest(self):
        url = os.path.join(self.__ep.geturl(), MANIFEST_PATH)
        # TODO: logging
        #log.einfo("Trying to get `{}'".format(url))
        # TODO Translate and rethrow a possible exception?
        contents = self.retrieve_remote_file(url)
        manifest = chewy.Manifest(contents)
        assert('We expect the repobase is a part of URL' and manifest.repobase.startswith(self.__ep.geturl()))
        return manifest


    def retrieve_remote_file(self, file_path):
        '''Retrieve the file specified'''
        try:
            self.__connection.request('GET', file_path)
            r = self.__connection.getresponse()
        except socket.error as ex:
            raise NetworkError("hostname: `{}'; port: `{}': {}".format(self.__ep.hostname, self.__ep.port, ex))
        # request should be read any way if we want reuse this connection
        data = r.read()
        if r.status != http.client.OK:
            raise RuntimeError(
                "Unable to retrieve a file `{}': {} - {}".format(file_path, r.status, r.reason)
              )
        return data.decode('utf-8')





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Chewy Manifest class
#

import urllib

import chewy
import chewy.meta as meta


class ManifestError(RuntimeError):
    pass


class Manifest(object):
    '''
        Class to represent a Chewy manifest
        contains fields (getters):
            repobase
            modules  - list<chewy.Module>
    '''

    _MIN_RECORD_FIELDS = 3

    def __init__(self, file_content):
        kvp_list = meta.parse(file_content)
        if not kvp_list:
            raise chewy.NoMetaError('No meta info found')

        # Default init members
        self.repobase = None

        # Validate meta info
        for kvp in kvp_list:
            if kvp[0] == meta.REPOBASE:
                if self.repobase is None:
                    self.repobase = kvp[1]
                else:
                    raise ModuleError('Multiple {} meta'.format(meta.REPOBASE))

        # Read modules list line by line
        lines = file_content.split('\n')
        self.modules = []
        line_number = 0
        for l in lines:
            line_number += 1
            line = l.lstrip()
            # Check for commented line
            if not l or l.startswith('#'):
                continue                                    # Ignore comments
            # Split record into fields
            record = line.split()
            if len(record) < self._MIN_RECORD_FIELDS:       # Minumum fields: path, version, description
                raise ManifestError('Too few fields in record at line {}'.format(line_number))

            # Unescape description
            # TODO Replace "magic number" w/ a constant
            record[2] = urllib.parse.unquote_plus(record[2])
            self.modules.append(chewy.Module(chewy.Module.PiecewiseConstruct(self.repobase, *record)))



#!/usr/bin/env python
#
# Install script for `chewy`
#

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import chewy


def readfile(filename):
    with open(filename, encoding='UTF-8') as f:
        return f.read()


setup(
    name             = 'chewy'
  , version          = chewy.__version__
  , description      = 'Python script to manage CMake modules'
  , long_description = readfile('README.md')
  , keywords         = 'CMake modules synchronizer'
  , maintainer       = 'Alex Turbov'
  , maintainer_email = 'I.zaufi@gmail.com'
  , url              = 'https://github.com/mutanabbi/chewy'
  , download_url     = 'https://github.com/mutanabbi/chewy/archive/version-{}.tar.gz'.format(chewy.__version__)
  , packages         = ['chewy']
  , scripts          = ['bin/chewy', 'bin/chewy-update-manifest']
  , license          = 'GNU General Public License v3 or later (GPLv3+)'
  , classifiers      = [
        'Development Status :: 5 - Production/Stable'
      , 'Environment :: Console'
      , 'Intended Audience :: Developers'
      , 'License :: OSI Approved :: GNU General Public License v3 (GPLv3)'
      , 'Natural Language :: English'
        # TODO Is it really Linux only?
      , 'Operating System :: POSIX :: Linux'
      , 'Programming Language :: Python'
        # TODO What about other Python versions?
      , 'Programming Language :: Python :: 3'
      , 'Topic :: Software Development :: Version Control'
      ]
  , install_requires = ['argparse', 'setuptools']
  , test_suite       = 'test'
  )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#


class FancyGrid(object):
    # TODO: Move this code to some kind of unit test
    # t = [[55555.666, 'aaa', '20', 'aaaaaaaa', 66], [42.5, 'bbbbbbbbbbbbb', '44444', 'bb', 112], [42, 'cc', 3, 'ccc', 555555555555]]
    # print(FancyGrid(t))

    def __init__(self, table):
        ''' Pass any sequence of sequences here '''
        # TODO: Support generators (iterable objects of any type)
        assert(hasattr(table, '__iter__') and hasattr(table[0], '__iter__'))
        # TODO: assert all raws contains same number of fields

        rows = len(table)
        cols = len(table[0])

        ## 1) get max size for columns from 1st till last - 1
        lens = [0 for x in range(cols - 1)]
        for i in table:
            for n in range(cols - 1):
                lens[n] = max(len(str(i[n])), lens[n])
        frmt = ('{{:<{}}}  ' * (cols - 1) + ' {{:<}}\n').format(*lens)

        self.__s = ''
        for i in table:
            self.__s += frmt.format(*i)

    def __str__(self):
        return self.__s



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Chewy Version class
#

import functools


class VersionError(RuntimeError):
    pass


@functools.total_ordering
class Version(object):

    def __init__(self, version):
        '''
            verstion: string by format MAJOR.MINOR (separated by .)
        '''
        try:
            self.major, self.minor = [int(i) for i in version.strip().split('.')]
        except ValueError as ex:
            raise VersionError from ex
        except AttributeError as ex:
            raise VersionError from ex

        if self.major == None or self.minor == None:
            raise VersionError('Incorrect version string: {}'.format(version))

        if self.major > 9999:
            raise VersionError('Major number too big: {}'.format(self.major))

        if self.minor > 9999:
            raise VersionError('Minor number too big: {}'.format(self.minor))


    def __lt__(self, other):
        return  self.major < other.major or (self.major == other.major and self.minor < other.minor)

    def __eq__(self, other):
        return (self.major, self.minor) == (other.major, other.minor)

    def __hash__(self):
        return self.major * 10000 + self.minor

    def __str__(self):
        return '{}.{}'.format(self.major, self.minor)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Chewy Module class
#

import chewy
import chewy.meta as meta


class ModuleError(RuntimeError):
    pass


class Module(object):
    '''
        Class to represent a Chewy module
        contain fields (getters):
            string repobase - repo URL and basedir
            string path     - relative to repobase file-path
            verstion        - string
            description     - string
            addons          - list<string>
    '''

    class PiecewiseConstruct(object):

        def __init__(self, repobase, path, version, description, *addons):
            self.addons = addons
            self.description = description
            self.path = path
            self.repobase = repobase
            self.version = chewy.Version(version)

    def __eq__ (self, other):
        '''
            Module is equal comparable, but note It isn't depended from module version
        '''
        return (self.repobase, self.path) == (other.repobase, other.path)


    def __init__(self, ctor_data):
        ''' Make a module from file content

            NOTE Unknown meta fileds are ignored
        '''
        # Default init members
        self.addons = []
        self.description = None
        self.path = None
        self.repobase = None
        self.version = None

        if isinstance(ctor_data, Module.PiecewiseConstruct):
            self.addons = ctor_data.addons
            self.description = ctor_data.description
            self.path = ctor_data.path
            self.repobase = ctor_data.repobase
            self.version = ctor_data.version
            return

        if not isinstance(ctor_data, str):
            raise TypeError('Module expect the only string parameter')

        kvp_list = meta.parse(ctor_data)
        if not kvp_list:
            raise chewy.NoMetaError('No meta info found')

        # Validate meta info
        for kvp in kvp_list:
            if kvp[0] == meta.ADDON:
                self.addons.append(kvp[1])

            if kvp[0] == meta.PATH:
                if self.path is None:
                    self.path = kvp[1]
                else:
                    raise ModuleError('Multiple {} meta'.format(meta.PATH))

            if kvp[0] == meta.VERSION:
                if self.version is None:
                    self.version = chewy.Version(kvp[1])
                else:
                    raise ModuleError('Multiple {} meta'.format(meta.VERSION))

            if kvp[0] == meta.DESCRIPTION:
                if self.description is None:
                    self.description = kvp[1]
                else:
                    raise ModuleError('Multiple {} meta'.format(meta.DESCRIPTION))

            if kvp[0] == meta.REPOBASE:
                if self.repobase is None:
                    self.repobase = kvp[1]
                else:
                    raise ModuleError('Multiple {} meta'.format(meta.REPOBASE))

        # Check that all attributes are here
        if self.path is None:
            raise ModuleError('Path is not defined for module')
        if self.version is None:
            raise ModuleError('Version is not defined for module')
        if self.description is None:
            raise ModuleError('Description is not defined for module')
        if self.repobase is None:
            raise ModuleError('RepoBase is not defined for module')


class ModuleStatus(object):
    '''Class to represent a module status.'''

    UNKNOWN = -1
    SAME_SAME = 0                                           # ;-)
    DELETED = 1
    UPDATE_AVAILABLE = 2
    LOCALY_MODIFIED = 3

    __STATUS_STRINGS = {
        SAME_SAME: ' * '
      , DELETED: ' D '
      , UPDATE_AVAILABLE: ' U '
      , LOCALY_MODIFIED: ' M '
    }


    def __init__(self, mod):
        self.module = mod
        self.__status = ModuleStatus.UNKNOWN
        self.__available_version = None


    def set_remote_version(self, remote_version):
        self.__available_version = remote_version
        if remote_version is None:
            self.__status = ModuleStatus.DELETED
        elif self.module.version < remote_version:
            self.__status = ModuleStatus.UPDATE_AVAILABLE
        elif self.module.version > remote_version:
            self.__status = ModuleStatus.LOCALY_MODIFIED
        else:
            self.__status = ModuleStatus.SAME_SAME


    def available_version(self):
        return self.__available_version


    def status_as_string(self):
        if self.__status in self.__STATUS_STRINGS:
            return self.__STATUS_STRINGS[self.__status]
        return '???'


    def needs_update(self):
        return ModuleStatus.UPDATE_AVAILABLE == self.__status



#!/usr/bin/env python
#
# Some tests for Chewy meta
#

""" Unit tests for chewy.meta """

# Standard imports
import os
import sys
import unittest

sys.path.insert(0, '..')

# Project specific imports
import chewy.meta

_test_case_1 = '''
# X-Chewy-Test: sample    \n
'''

_test_case_2 = '''
# X-Chewy-Test: sample

some text

  #    X-Chewy-Test-2:    sample 2
'''

class ChewyMetaTester(unittest.TestCase):
    '''Unit tests for [???]'''

    def setUp(self):
        pass

    def test_empty_meta(self):
        result = chewy.meta.parse("")
        self.assertEqual(len(result), 0)

    def test_empty_meta_2(self):
        result = chewy.meta.parse("#\n#   \n   #")
        self.assertEqual(len(result), 0)

    def test_meta(self):
        result = chewy.meta.parse(_test_case_1)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0][0], "Test")
        self.assertEqual(result[0][1], "sample")

    def test_meta_2(self):
        result = chewy.meta.parse(_test_case_2)
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0][0], "Test")
        self.assertEqual(result[0][1], "sample")
        self.assertEqual(result[1][0], "Test-2")
        self.assertEqual(result[1][1], "sample 2")


if __name__ == '__main__':
    suite = unittest.makeSuite(ChewyMetaTester)
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    if result.wasSuccessful() != True:
        sys.exit(1)

